# $Id: Foo2Mail.pm,v 1.9 2003/03/30 10:55:22 struan Exp $

package Foo2Mail;
use strict;
use IO::File;
use File::Spec;
use File::Path qw(mkpath);
use MIME::Lite;

BEGIN {
	use vars qw( $VERSION $ERROR );
	$VERSION     = 0.04;
}

my ( @REQUIRED, @ALL, %DEFAULTS );

@REQUIRED = qw(fetch to_fetch to);
@ALL = ( @REQUIRED, qw(from subject base_dir mail_each mail_items 
                      delete_after no_auto no_send include_errors 
                      warn_errors _last_name _last_target item_delete_after 
                      always_new ) );

%DEFAULTS = (
    from                =>  'f2m@exo.org.uk',
    subject             =>  'Content from Foo2Mail',
    base_dir            =>  './.foo2mail', # this is prob a bad idea
    mail_each           =>  0,
    mail_items          =>  0,
    delete_after        =>  30,
    item_delete_after   =>  {},
    no_auto             =>  0,
    no_send             =>  0,
    include_errors      =>  0,
    warn_errors         =>  1,
    always_new          =>  0,
    _last_name          =>  '',
    _last_target        =>  '',
);

=head1 NAME

Foo2Mail - Takes the tedium out of automatically fetching and mailing content

=head1 SYNOPSIS

    use Foo2Mail;

    # use LWP::Simple::get to fetch from url
    # use quick year/month/day date as id
            
    sub fetch {
        my ($self, $name, $target) = @_;
        my $content;
        if ($content = get($target)) {
            return {
                name => $name,
                content => $content,
                id  =>  (localtime())[5,4,3],
            };
        } else {
            $self->error('Failed to fetch $name ($target)');
            return undef;
        }
    }

    my %to_fetch = (
        a_site  =>  'http://example.com/example.html',
        b_site  =>  'http://example.com/example2.html',
    );

    my $f2m = Foo2Mail->new(
                fetch       =>  \&fetch,
                to_fetch    =>  \%to_fetch,
                to          =>  'user@example.com',
                from        =>  'f2m@example.com',
                subject     =>  'Foo2Mail output',
    ) or die $Foo2Mail::ERROR;


=head1 DESCRIPTION

Foo2Mail makes turning non mail content into an email easy. You provide a list of content you want turned into an email and a means of fetching that content and it will do the rest.

By default it uses a simple system of directories and files to record fetched content. However it is designed to be subclassed so you can use a database to store information or use your templating system of choice to format the email. Foo2Mail::Template which uses the Template Toolkit for formatting is included as an example.

=head1 USAGE

The example in the synopsis is the simplest way to use Foo2Mail. In that case user@example.com will recieve a single email the content of which looks something like this:

    From: f2m@example.com
    To: user@example.com
    Subject: Foo2Mail output

    a_site
    ----------------------------------------------------------------------
    content of http://example.com/example.html

    b_site
    ---------------------------------------------------------------------
    content of http://example.com/example2.html

In order to prevent content being sent twice Foo2Mail uses a simple system of directories and files to keep track of what's been sent. By default it will create a directory for each key in the hash. Within this will be one file per fetched piece of content, the filename being that of the Id. So, for the Synopsis example the data directory will have two directories:

    a_site
    b_site

each containg a file each called something like:

    20030315

indicating that content was fetched from a_site and b_site at that date. Any attempt to send content from those sites on that date will not produce an email. It should be noted that Foo2Mail does some sanity checking on strings it will use as filenames. It's not very complex so if possible Ids should be something suitable for use as a filename. 

By default Foo2Mail will clean out files older than 30 days from these directories. There are options to fine tune this on a global and per target basis.

However, Foo2Mail is capable of a great deal more. You may find that putting all your content in a single email is limiting. In this case if you turn on the mail_each option you will get an email for each target. If in the synopsis example we call Foo2Mail thus:

    my $f2m = Foo2Mail->new(
                fetch       =>  \&fetch,
                to_fetch    =>  \%to_fetch,
                to          =>  'user@example.com',
                from        =>  'f2m@example.com',
                subject     =>  'Foo2Mail output',
                mail_each   =>  1,
    ) or die $Foo2Mail::ERROR;

then two emails will be generated:

    From: f2m@example.com
    To: user@example.com
    Subject: Foo2Mail output: a_site

    content of http://example.com/example.html

    From: f2m@example.com
    To: user@example.com
    Subject: Foo2Mail output: b_site

    content of http://example.com/example2.html

Of course, it may be the case that within each target there are multiple different items that you wish to seperate out (e.g an RSS feed). This can be done by having the fetch routine return a reference to an array of item hashes:

    # WARNING Will Robinson: very artificial example
    # assume content consists of lines similar to
    # Robert Smith:My Dog Spot:Richard James:A fine declaration from Mr James
    # Julian Cope:Programming Perl:Larry Wall et al:Not enough ley lines

    sub fetch {
        my ($self, $name, $target) = @_;
        my @reviews;
        open TARGET "<$target" or do {
            $self->error("Failed to open $name ($target: $!)");
            return undef;
        };
        my @content = <TARGET>;
        close TARGET;
        chomp(@content);
        while (@content) {
            my ($reviewer, $book, $author, $review) = split(':');
            push @reviews, {
                name    =>  $name,
                subject =>  $book,
                content =>  "$book by $author\nReviewed by $reviewer\n------\n$review\n",
                id      =>  "$book_$reviewer",
            };
        }
        return \@reviews;
    }

    my %to_fetch = (
        reviews =>  '/usr/local/share/reviews.txt',
    );

    my $f2m = Foo2Mail->new(
                fetch       =>  \&fetch,
                to_fetch    =>  \%to_fetch,
                to          =>  'user@example.com',
                from        =>  'f2m@example.com',
                subject     =>  'Foo2Mail output',
    ) or die $Foo2Mail::ERROR;

In this case the user will get an email like this:

    From: f2m@example.com
    To: user@example.com
    Subject: Foo2Mail output

    reviews
    ----------------------------------------------------------------------
    My Dog Spot by Richard James
    Reviewed by Robert Smith
    -------
    A fine declaration from Mr James

    Programming Perl by Larry Wall et al
    Reviewed by Julian Cope
    -------
    Not enough ley lines

and the data directory will contain:

    reviews/My_Dog_Spot_Robert_Smith
    reviews/Programming_Perl_Julian_Cope

If, however, you want to send one email per item then you can use the mail_items option to new:

    my $f2m = Foo2Mail->new(
                fetch       =>  \&fetch,
                to_fetch    =>  \%to_fetch,
                to          =>  'user@example.com',
                from        =>  'f2m@example.com',
                subject     =>  'Foo2Mail output',
                mail_items  =>  1,
    ) or die $Foo2Mail::ERROR;

In this case there will be one email sent for each line of the processed file, the subject of which will be the title of the book.

Foo2Mail can also be used just to create and format the emails so that if you want deliver the actual emails by some other means (some sort of mailing list software for example), you can access the raw emails. To do this then you can use the no_email option to new and then access the emails via the get_email method.

=head1 METHODS

If you just want Foo2Mail to fetch and deliver then you should only ever need to use the new method.

=head2 new

Creates a Foo2Mail object. Either returns the new Foo2Mail object or undef. If it fails then the error can be accessed via $Foo2Mail::ERROR. Takes the following options:

=over 4

=item fetch

Reference to a subroutine that will fetch the content. It should take three arguments, self, name and target in that order, which are a reference to the current Foo2Mail object, the descriptive name you gave the item to fetch and the information required by C<fetch()> to fetch the associated content, e.g a URI. 

The function should either return a hash reference as shown in the SYNOPSIS above where name and content are as expected and id is a value that uniquely identifies that piece of content (e.g the comment id of a slashdot comment). If more than one piece of content is associated with a target (e.g if the target is an RSS feed) then a reference to array of the previously described hashes should be returned. Optionally the hash reference can include a subject key to be used in conjunction with the mail_items option.

If the fetch fails then undef should be returned.

Any errors encountered can and should be passed back to the Foo2Mail object with the error method. This will enable the errors to be included in the email as well as on STDERR.

NB: Foo2Mail will run this code under eval so you cannot use C<die> in the case of a fatal error. You should instead use the fatal_error method of Foo2Mail which will ensure that processing stops.

=item to_fetch

Reference to a hash of name => target pairs that associate a descriptive name with the location of content to pass to C<fetch>.

=item to

Either a single email address or an array reference of email address(es) to send the content to.

=item from

Email address the mail should appear to come from. This is optional and if not supplied then the default of f2m@exo.org.uk will be used.

B<NB>: The default email address is not guaranteed to be valid in perpetuity but while it is a functional email address all email send to it will be discarded.

=item subject

The subject to use for the email. This is optional and if not supplied the the default of 'Content from Foo2Mail' will be used. It will be overridden if the mail_each or mail_items options are set. 

If mail_each is set then the subject will C<$subject: $key> where $subject is the value of this option and $key is the value of the key from to_fetch associated with the fetched item.

If mail_items is set then if a subject key is present in the items from the fetch function this will be used otherwise the value of this option will be used.

=item base_dir

Path to directory to save information about content sent. Defaults to './.foo2mail'.

=item mail_each

If true then will create a mail for each key in the to_fetch hash. Defaults to false.

=item mail_items

If true then a mail will be created for each item of content returned by the fetch function. Defaults to false.

=item delete_after

Number of days to store information about what has been delivered. If set to 0 then cleanup never occurs. Defaults to 30. 

=item item_delete_after

Hash reference to alter above on a per item basis. Keys should match keys in to_fetch argument. If a target is not present here then the default will be used. This will be ignored if delete_after is set to 0.

=item no_auto

If true then Foo2Mail will only instantiate and not automatically start fetching and sending content. Defaults to false.

=item no_send

If true then email is not sent. Instead the email can be accessed by calling the C<get_email> method. Defaults to false.

NB: Setting this to true means that C<cleanup> will not be called automatically.

=item always_new

If true then all items are assumed to be new and will be sent. Defaults to false.

=item include_errors

If true then errors that occurred in processing will be included at the end of the email. Defaults to false.

=item warn_errors

If true then as well as being kept track of for inclusion in the email errors are printed out to STDERR. Defaults to true.

=back

=cut


sub new {
	my ( $class, %parameters ) = @_;

	my $self = bless ( {}, ref($class) || $class );

    $self->init(%parameters) or return undef;

    $self->process() unless $self->{no_auto};

    $self->{_fatal} ? return undef : return $self;
}

=head2 init

Actually processes the options passed to new. Checks that it has all the options required and sets the defaults for the optional ones.

=cut

sub init {
    my $self = shift;

    $ERROR = '';
    
    my %params = @_;

    # do some sanity checks
    my @missing = grep { !defined $params{$_} } @REQUIRED;

    if ( @missing ) {
        $self->fatal_error( "Required params missing: " . join(', ', @missing) );
        return undef;
    }

    unless ( ref $params{fetch} eq 'CODE' ) {
        $self->fatal_error("Parameter fetch is not a code reference");
        return undef;
    }

    unless ( scalar keys %{$params{to_fetch}} > 0) {
        $self->fatal_error( "Parameter to_fetch must have at least one name, target pair" );
        return undef;
    }

    $params{to} = [ $params{to} ] unless ref $params{to} eq 'ARRAY';

    unless ( scalar @{$params{to}} > 0 and $params{to}->[0] ) {
        $self->fatal_error("Parameter to must have at least one email address");
        return undef;
    }

    # set values
    foreach my $param ( @ALL ) {
        $self->{$param} = defined $params{$param} ? $params{$param} 
                                                  : $DEFAULTS{$param};
    }

    $self;
}

=head2 process

Main loop for dealing with fetching, checking, storing and sending all the information. Called at the end of object initialisation unless no_auto parameter set.

In essence for each entry in %to_fetch it attempts to fetch the content. If successful and the content is new it stores the content in memory.

After this a mail object (MIME::Lite by default) is generated for each mail. This included formatting the email body. How many mails are generated is determined by whether mail_each or mail_items is set.

If no_send is set processing will stop at this point.

It then attempts to send each mail and if succesful saves that mail has been sent.

Old information is then cleaned up.

If C<fatal_error> is called then process will stop at the first convenient point and undef will be returned. The fatal error can be accessed via $f2m->error();.

=cut

sub process {
    my $self = shift;

    foreach my $name ( keys %{$self->{to_fetch}} ) {
        return undef if $self->{_fatal};

        while ( my $info = $self->fetch($name, $self->{to_fetch}->{$name}) ) {
            return undef if $self->{_fatal};
            if ( $info and ( $self->{always_new} or $self->is_new($info) ) ) {
                $self->add_to_content($info);
            }
        }
    }

    $self->make_mails() or return undef;
    return if $self->{no_send};

    $self->send() or return undef;

    $self->cleanup() or return undef;
}

=head2 fetch

Calls the C<fetch> method passed in. The called method is run under eval in order to catch errors. 

If the user supplied fetch function returns multiple items then they are accessed by repeated calls to fetch.

Fetch either returns a hash reference as per the fetch function supplied by the user or undef if either no content was fetched or all fetched content has been accessed. 

=cut

# the _last_name and _last_target values are there to stop infinite loops
# without them, after returning all the data that it fetched, the next call
# to fetch sees that _raw_data is undef so it tries to fetch some content.
# however as it's not returned false to process() it still has the name and
# target that were last passed so it fetches them again and bingo! an
# infinite loop. hence the check against _last_name and _last_target

sub fetch {
    my $self = shift;
    my ( $name, $target ) = @_;
    my $values;
   
    # we've already fetched and stored content so return next in list
    if ($self->{_raw_data}) {
        $values = shift @{$self->{_raw_data}};
        # [] is true so need to set to undef explicitly
        $self->{_raw_data} = undef if scalar @{$self->{_raw_data}} == 0;
    } else {
        # we've already fetched this
        return undef if $self->{_last_name} eq $name and
                        $self->{_last_target} eq $target;

        eval {
            $values = $self->{fetch}->($self, $name, $target);
        };
        if ($@) {
            $self->error($@);
            return undef;
        }
        
        if (ref $values eq 'ARRAY') {
            $self->{_raw_data} = $values;
            $values = shift @{$self->{_raw_data}};
        }
    }
    # keep track of what we last fetched to prevent infinite loops
    $self->{_last_name} = $name;
    $self->{_last_target} = $target;
    return $values;
}

=head2 is_new

Takes the hash reference returned by C<fetch> and checks to see if it has been delivered. If it has then 0 is returned else 1.

=cut

sub is_new {
    my $self = shift;
    my $values = shift;
    
    my ($name, $id) = ($self->_to_filename($values->{name}), 
                       $self->_to_filename($values->{id}));
                   
    # need to test each bit individually otherwise intermediate stages
    # will be autovivified.
    return 0 if exists $self->{_fetched_info} &&
                exists $self->{_fetched_info}->{$values->{name}} &&
                exists $self->{_fetched_info}->{$values->{name}}->{$values->{id}};
                
    my $file = File::Spec->catpath(
        '', 
        File::Spec->catdir($self->{base_dir}, $name), 
        $id
    );

    chomp($file);
    return 0 if -e $file;
    return 1;
}

=head2 add_to_content

Takes the hash reference returned by C<fetch> and adds it the the content to be included in the email. 

=cut

sub add_to_content {
    my $self = shift;
    my $values = shift;
    push @{$self->{_fetched_info}->{$values->{name}}}, 
         {
            id => $values->{id}, 
            subject => $values->{subject} || undef, 
            content => $values->{content}
        };
}

=head2 format

Takes the fetched content and formats it into an email body and returns the result.

=cut

sub format {
    my $self = shift;
    my $mail = shift;

    my $body;
    
    foreach my $section (@{$mail->{sections}}) {
        $body .= $section->{name} . "\n" . '-' x 72 . "\n" if $section->{name};
        foreach my $items ( @{$section->{items}} ) {
            $body .= $items->{content} ."\n";
        }
    }
    
    if ($self->{include_errors} and $self->{_error_list}) {
        $body .= "\nErrors during processing\n" . '-' x 72 . "\n";
        $body .= join("\n", @{$self->{_error_list}}) ."\n";
    }

    return $body;       
}

=head2 make_mails

Takes the contents of _fetched_info and splits it up into an array of seperate mails according to whether mail_each or mail_items is set. Then each mail is formatted and a MIME::Lite object is generated for each one.

=cut

sub make_mails {
    my $self = shift;

    my @mails;

    # if we haven't fetched anything then we don't need to do anything
    return undef unless $self->{_fetched_info};

    # something to upset the perl haters out there...
    # munges _fetched_info into the structure format et al expects.
    if ($self->{mail_each}) {
        @mails = map { +{
                            name    =>  $_,
                            subject => $self->{subject} . ": $_",
                            sections => [
                                +{ 
                                    items =>  $self->{_fetched_info}->{$_} 
                                }
                            ]
                        }
                     } keys %{$self->{_fetched_info}};
    } elsif ($self->{mail_items}) {
        @mails = map { 
                        my $name = $_;
                        map {
                                +{
                                    name => $name,
                                    id   => $_->{id},
                                    sections => [
                                        +{ 
                                            items => [$_] 
                                        }
                                    ],
                                    subject => $_->{subject} 
                                }
                            } @{$self->{_fetched_info}->{$_}} 
                        } keys %{$self->{_fetched_info}};
    } else {
        @mails = (
                    +{
                        sections => [
                            map { 
                                    +{
                                        name    =>  $_,
                                        items   =>  $self->{_fetched_info}->{$_}
                                    }
                                } keys %{$self->{_fetched_info}}
                        ]
                    }
                 );
    }
    foreach my $mail (@mails) {
        my $body = $self->format($mail);
        next unless $body;
        push @{$self->{_mail_objs}}, { 
                id      => $mail->{id} ? $mail : undef, 
                name    => $mail->{name},
                obj     => $self->_generate_mail($mail, $body) 
            };
    }

    return undef if $self->{_fatal};
    return undef unless scalar @mails;
}

=head2 send

Sends the email and stores it if sending is sucessful. Returns 1 if sent or 0 otherwise.

=cut

sub send {
    my $self = shift;

    foreach my $mail (@{$self->{_mail_objs}}) {
        next unless $mail;
        eval {
            $mail->{obj}->send();
        };
        if ($@) {
            $self->error("Problem sending mail: $@");
            next;
        } else {
            $self->store(name => $mail->{name}, id => $mail->{id});
        }
    }
    return 1;
}

=head2 store

Stores the list of content successfully fetched and sent. By default it will store all the information in _fetched_info. It does accept two named arguments: id and name.

name will cause it to only save the entries under that key.

id will cause it to only save that id. id should be a fetched item hashref as retuend by the fetch function. name must also be provided in this case.

    $self->store(); # store everything

    $self->store(name => 'foo'); # store everything fetched under the foo key
                                 # in to_fetch
                                 
    # store the key $item fetched under the foo key.
    $self->store(name => 'foo', id => $item);

=cut

sub store {
    my $self = shift;
    my %args = @_;

    if ($args{id}) {
        $self->_store_id($args{id}, '', $args{name});
    } elsif ($args{name}) {
        $self->_store_name($args{name});
    } else {
        foreach my $name (keys %{$self->{_fetched_info}}) {
            $self->_store_name($name);
        }
    }
}

=head2 cleanup

Deletes old information about information sent. By default will remove any information older than 30 days.

=cut

sub cleanup {
    my $self = shift;
    
    return if $self->{delete_after} == 0;

    foreach my $name (keys %{$self->{_fetched_info}}) {
        my $dir = File::Spec->catdir($self->{base_dir},
                                     $self->_to_filename($name));

        foreach my $file (glob "$dir/*") {
            my $days = exists $self->{item_delete_after}->{$name} 
                            ? $self->{item_delete_after}->{$name}
                            : $self->{delete_after};
            unlink $file if -M $file > $days;
        }
    }
}

=head2 get_email 

Returns stringified emails. i.e. the complete text of the email including headers and body.

If one email has been generated returns the text of that. If multiple were generared returns an array of the text of all of them.

=cut

sub get_email {
    my $self = shift;

    if ($self->{_mail_objs}) {
        return $self->_stringify_emails();
    } else {
        $self->make_mails() or return undef;
        return $self->_stringify_emails();
    }
}

=head1 INTERNALS

This information is intended for people who want to subclass the module in order to override either the formating mechanism or the data storage mechanism.

=head2 Fetched Information

After information has been fetched it is stored internally in a hash of array refs and this is the information used by the store function. It is stored in the _fetched_info key of the hash that represents the object and takes the following form:

    $self->{_fetched_info} = {
        name1   =>  [
                        { id => 'id1', content => 'content'},
                        { id => 'id2', content => 'content'},
                    ],
        name2   =>  [
                        {id => 'id1', content => 'content'},
                        {id => 'id2', content => 'content'},
                    ],
    };

Any code implementing C<add_to_content> must also maintain this structure.

This structure is used by C<make_mails> to generate the array of mails used by format. This structure takes the following form:

    @mails = (
               {
                  id        => 'item id'    # optional
                  name      => 'name',      # optional
                  subject   => 'subject',   # optional
                  sections  => [
                                 {
                                    name  => 'section name', # optional
                                    items => {}  # &fetch item hashref
                                 }
                               ]
               }
    );

The idea of this is that each mail is made up of, optionally named, sections, each containing one or more items. These correlate to the keys of to_fetch and the items fetched under them.

The outer id is used internally for saving items when mail_items is set. The outer name is similarly used when mail_items and mail_each are set. The subject is used as imagined.

The inner name is used in the default C<format> for splitting up content from the different keys of to_fetch.

=head2 Email Body

The C<format> method will be passed an element of the array above and should return the formatted email body.

The content of the C<_error_list> key should also be checked and included if the include_errors key is true. The _error_list key is an array reference of all the error messages that have occurred during processing.

=head1 ERROR HANDLING

The two methods described below should be used when errors occur in preference to ad-hoc calls to warn or die.

=head2 error

    $f2m->error('something went wrong');
    # or
    my $error = $f2m->error();

Takes an error message as an argument and adds that error message to the list of those generated. Also prints out a message to STDERR if the warn_errors flag is set.

If no error is passes then it will return the last error generated.

=cut


sub error {
    my $self = shift;
    my $error = shift;

    if ($error) {
        push(@{$self->{_error_list}}, $error);
        warn $error if $self->{warn_errors};
    } else {
        return $self->{_error_list}->[$#{$self->{_error_list}}];
    }
}

=head2 fatal_error

    $f2m->fatal_error('something very bad happened');

Stops processing as soon as possible. Will always send error to STDERR. Will not generate an email though unless occurs after C<send> has been called. Also takes error message as an argument.

=cut

sub fatal_error {
    my $self = shift;
    my $error = shift;

    push(@{$self->{_error_list}}, $error);
    $ERROR = $error;
    warn $error if $self->{warn_errors};

    $self->{_fatal} = 1;
}

=head1 OTHER INTERNAL FUNCTIONS

=head2 _generate_mail

Generates the email. Really just a wrapper round instantiation of a MIME::Lite object. The resulting MIME::Lite object is stored in the C<_mail_obj> key.

Will return undef if cannot create the mail object.

=cut

sub _generate_mail {
    my $self = shift;
    my $mail = shift;
    my $body = shift;
    my $mail_obj;

    unless ( $body ) {
        $self->error("Email body is empty");
        return undef;
    }

    $mail_obj = MIME::Lite->new(
        To      =>  join(',',@{$self->{to}}),
        From    =>  $self->{from},
        Subject =>  $mail->{subject} || $self->{subject},
        Type    =>  'TEXT',
        Data    =>  $body,
    ) or do {
        $self->fatal_error("Problem initalising mail object");
        return undef;
    };

    return $mail_obj;
}

=head2 _store_name

Takes the name of a key in to_fetch/_fetched_info as an argument and stores that each entry under that key has been send.

=cut

sub _store_name {
    my $self = shift;
    my $name = shift;

    my $dir = $self->_get_name_dir($name);
                             
    my @name = @{$self->{_fetched_info}->{$name}};
    foreach my $id ( @name ) {
        $self->_store_id($id, $dir);
    }
}

=head2 _store_id

Takes a mail item and a directory as arguments and stores that id as being send. If the directory is undefined then it tries to generate one from the optional third name argument

=cut

sub _store_id {
    my $self = shift;
    my $id = shift;
    my $dir = shift || $self->_get_name_dir(shift);

    return unless $id->{id};
    return 1 if $id->{stored};
    my $file = $self->_to_filename($id->{id});
    my $path = File::Spec->catpath('',$dir, $file);
    my $fh = IO::File->new($path, O_CREAT | O_EXCL) or do {
        $self->error("Problem storing info: Couldn't create file $path [$!]");
        return undef;
    };
    $id->{stored} = 1;
}

=head2 _get_name_dir 

Takes a key name from to_fetch/fetched_info and returns the directory to store information about that in

=cut

sub _get_name_dir {
    my $self = shift;
    my $name = shift;

    my $dir = File::Spec->catdir($self->{base_dir},
                                     $self->_to_filename($name));

    $dir .= '/' unless $dir =~ m#/$#;

    unless (-e $dir) {
        eval { mkpath($dir); };
        if ($@) {
            $self->error("Problems storing info: Couldn't create directory $dir [$@]");
            return undef;
        }
    }
    
    return $dir;
}

=head2 _stringify_emails

Returns either an array of stringified emails, a stringified email if there is only one or an empty string if there are none.

=cut

sub _stringify_emails {
    my $self = shift;

    if (scalar @{$self->{_mail_objs}} == 1) {
        return $self->{_mail_objs}->[0]->{obj}->as_string();
    } elsif ($self->{_mail_objs}) {
        my @mails = map { $_->{obj}->as_string() } @{$self->{_mail_objs}};
        return \@mails;
    } else {
        return '';
    }
}

=head2 _to_filename

Takes a string and makes it into something approximating a valid filename. It's really very crude.

=cut

sub _to_filename {
    my $self = shift;
    my $string = shift || '';

    chomp($string);
    $string =~ s#[^\w\-\.]#_#g;

    return $string;
}

1; 
__END__

=head1 BUGS

It's a little memory intensive. Given that by default it stores all the information fetched this is not suprising. Munging all that information into a different structure for formatting is deeply non optimal though.

=head1 SUPPORT

Author email.

=head1 AUTHOR

Struan Donald
<modules@exo.org.uk>
http://www.exo.org.uk/code/

Some of the basic concepts were stolen from Richard Clamp's j2m script.

=head1 COPYRIGHT

Copyright (C) 2002-2003 Struan Donald. All rights reserved.

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1), MIME::Lite.

=cut

